!********************************************************************************
!MODULE SIM--------------------------------------------------- BY SML. 14/08/2012
!********************************************************************************

 MODULE SIM

!********************************************************************************
!THIS MODULE CONTAINS SUBROUTINES "ODRV" & "SDRV" (AND AUXILLIARY SUBROUTINES MD, 
!MDI, MDM, MDP, MDU, SRO, SSF, SNF AND SNS) FROM THE YALE'S SPARSE MATRIX PACKAGE
!(YSMP), DEVELOPED BY THE DEPARTMENT OF COMPUTER SCIENCE, YALE UNIVERSITY. SOLVES 
!SPARSE POSITIVE DEFINITE SYSTEM OF EQUATIONS STORED IN IJA FORMAT USING GAUSSIAN
!ELIMINATION WITHOUT PIVOTING.
!********************************************************************************
!
!***VERSION 1.1***
!AUXILIARY MODULE "PARAM" WITH NUMERICAL PRECISION DEFINITION IS REQUIRED.
!***
!
!-------
!MODULES
!-------
 USE PARAM
!---
!
 CONTAINS
!															  15/01/1981!SIM00010
!***********************************************************************!SIM00020
 SUBROUTINE  ODRV (N, IA, JA, A, P, IP, NSP, ISP, PATH, FLAG)			!SIM00030
!***********************************************************************!SIM00040
!DRIVER FOR SPARSE MATRIX REORDERING ROUTINES.							!SIM00050
!												                        !SIM00060
!-----------                                                            !SIM00070
!DESCRIPTION															!SIM00080
!-----------                                                            !SIM00090
!ODRV USES A MINIMUM DEGREE ORDERING ALGORITHM TO PERFORM ROWS AND CO-	!SIM00100
!LUMNS PERMUTATIONS OF A SYMMETRIC MATRIX M STORED IN (IA,JA,A) FORMAT.	!SIM00110
!FOR THE REORDERED MATRIX, THE WORK AND STORAGE REQUIRED TO PERFORM		!SIM00120
!GAUSSIAN ELIMINATION IS (USUALLY) SIGNIFICANTLY LESS.					!SIM00130
!                                                                       !SIM00140
!IF ONLY THE NONZERO ENTRIES IN THE UPPER TRIANGLE OF M ARE BEING		!SIM00150
!STORED, THEN ODRV SYMMETRICALLY REORDERS (IA,JA,A), [OPTIONALLY]		!SIM00160
!WITH THE DIAGONAL ENTRIES PLACED FIRST IN EACH ROW. THIS IS TO			!SIM00170
!ENSURE THAT IF M(I,J) WILL BE IN THE UPPER TRIANGLE OF M WITH			!SIM00180
!RESPECT TO THE NEW ORDERING, THEN M(I,J) IS STORED IN ROW I (AND		!SIM00190
!THUS M(J,I) IS NOT STORED); WHEREAS IF M(I,J) WILL BE IN THE			!SIM00200
!STRICT LOWER TRIANGLE OF M, THEN M(J,I) IS STORED IN ROW J (AND		!SIM00210
!THUS M(I,J) IS NOT STORED).											!SIM00220
!                                                                       !SIM00230
!--------------------------                                             !SIM00240
!STORAGE OF SPARSE MATRICES												!SIM00250
!--------------------------						                        !SIM00260
!THE NONZERO ENTRIES OF THE MATRIX M ARE STORED ROW-BY-ROW IN THE		!SIM00270
!ARRAY A. TO IDENTIFY THE INDIVIDUAL NONZERO ENTRIES IN EACH ROW,		!SIM00280
!WE NEED TO KNOW IN WHICH COLUMN EACH ENTRY LIES. THESE COLUMN			!SIM00290
!INDICES ARE STORED IN THE ARRAY JA; I.E., IF A(K) = M(I,J), THEN		!SIM00300
!JA(K) = J. TO IDENTIFY THE INDIVIDUAL ROWS, WE NEED TO KNOW WHERE		!SIM00310
!EACH ROW STARTS. THESE ROW POINTERS ARE STORED IN THE ARRAY IA;		!SIM00320
!I.E., IF M(I,J) IS THE FIRST NONZERO ENTRY (STORED) IN THE I-TH ROW	!SIM00330
!AND A(K) = M(I,J), THEN  IA(I) = K. MOREOVER, IA(N+1) POINTS TO		!SIM00340
!THE FIRST LOCATION FOLLOWING THE LAST ELEMENT IN THE LAST ROW.			!SIM00350
!THUS, THE NUMBER OF ENTRIES IN THE I-TH ROW IS  IA(I+1) - IA(I),		!SIM00360
!THE NONZERO ENTRIES IN THE I-TH ROW ARE STORED CONSECUTIVELY IN		!SIM00370
!                                                                       !SIM00380
!            A(IA(I)),  A(IA(I)+1),  ..., A(IA(I+1)-1),                 !SIM00390
!                                                                       !SIM00400
!AND THE CORRESPONDING COLUMN INDICES ARE STORED CONSECUTIVELY IN		!SIM00410
!                                                                       !SIM00420
!            JA(IA(I)), JA(IA(I)+1), ..., JA(IA(I+1)-1).                !SIM00430
!                                                                       !SIM00440
!SINCE THE COEFFICIENT MATRIX IS SYMMETRIC, ONLY THE NONZERO ENTRIES	!SIM00450
!IN THE UPPER TRIANGLE NEED BE STORED. FOR EXAMPLE, THE MATRIX			!SIM00460
!                                                                       !SIM00470
!             ( 1  0  2  3  0 )                                         !SIM00480
!             ( 0  4  0  0  0 )                                         !SIM00490
!         M = ( 2  0  5  6  0 )                                         !SIM00500
!             ( 3  0  6  7  8 )                                         !SIM00510
!             ( 0  0  0  8  9 )                                         !SIM00520
!                                                                       !SIM00530
!COULD BE STORED AS														!SIM00540
!                                                                       !SIM00550
!            | 1  2  3  4  5  6  7  8  9 10 11 12 13                    !SIM00560
!         ---+--------------------------------------                    !SIM00570
!         IA | 1  4  5  8 12 14                                         !SIM00580
!         JA | 1  3  4  2  1  3  4  1  3  4  5  4  5                    !SIM00590
!          A | 1  2  3  4  2  5  6  3  6  7  8  8  9                    !SIM00600
!                                                                       !SIM00610
!OR (SYMMETRICALLY) AS													!SIM00620
!                                                                       !SIM00630
!            | 1  2  3  4  5  6  7  8  9                                !SIM00640
!         ---+--------------------------                                !SIM00650
!         IA | 1  4  5  7  9 10                                         !SIM00660
!         JA | 1  3  4  2  3  4  4  5  5                                !SIM00670
!          A | 1  2  3  4  5  6  7  8  9          .                     !SIM00680
!                                                                       !SIM00690
!                                                                       !SIM00700
!PARAMETERS:															!SIM00710
!                                                                       !SIM00720
!    N    - ORDER OF THE MATRIX                                         !SIM00730
!                                                                       !SIM00740
!    IA   - INTEGER VECTOR CONTAINING POINTERS TO DELIMIT ROWS IN JA	!SIM00750
!           AND A; DIMENSION = N+1										!SIM00760
!                                                                       !SIM00770
!    JA   - INTEGER VECTOR CONTAINING THE COLUMN INDICES CORRESPONDING	!SIM00780
!           TO THE ELEMENTS OF A; DIMENSION = NUMBER OF	NONZERO ENTRIES	!SIM00790
!           IN (THE UPPER TRIANGLE OF) M								!SIM00800
!                                                                       !SIM00810
!    A    - REAL VECTOR CONTAINING THE NONZERO ENTRIES IN (THE UPPER	!SIM00820
!           TRIANGLE OF) M, STORED BY ROWS; DIMENSION = NUMBER OF		!SIM00830
!           NONZERO ENTRIES IN (THE UPPER TRIANGLE OF) M				!SIM00840
!                                                                       !SIM00850
!    P    - INTEGER VECTOR USED TO RETURN THE PERMUTATION OF THE ROWS	!SIM00860
!           AND COLUMNS OF M CORRESPONDING TO THE MINIMUM DEGREE		!SIM00870
!           ORDERING; DIMENSION = N										!SIM00880
!                                                                       !SIM00890
!    IP   - INTEGER VECTOR USED TO RETURN THE INVERSE OF THE			!SIM00900
!           PERMUTATION RETURNED IN P; DIMENSION = N					!SIM00910
!                                                                       !SIM00920
!    NSP  - DECLARED DIMENSION OF THE VECTOR ISP; NSP MUST BE AT LEAST	!SIM00930
!           3N+4K, WHERE K IS THE NUMBER OF NONZEROES IN THE STRICT		!SIM00940
!           UPPER TRIANGLE OF M											!SIM00950
!                                                                       !SIM00960
!    ISP  - INTEGER VECTOR USED FOR WORKING STORAGE; DIMENSION = NSP    !SIM00970
!                                                                       !SIM00980
!    PATH - INTEGER PATH SPECIFICATION; VALUES AND THEIR MEANINGS ARE:	!SIM00990
!                                                                       !SIM01000
!           1  FIND MINIMUM DEGREE ORDERING ONLY.						!SIM01010
!           2  FIND MINIMUM DEGREE ORDERING AND REORDER SYMMETRICALLY	!SIM01020
!              STORED MATRIX (USED WHEN ONLY THE NONZERO ENTRIES IN		!SIM01030
!              THE UPPER TRIANGLE OF M ARE BEING STORED).				!SIM01040
!           3  REORDER SYMMETRICALLY STORED MATRIX AS SPECIFIED BY		!SIM01050
!              INPUT PERMUTATION (USED WHEN AN ORDERING HAS ALREADY		!SIM01060
!              BEEN DETERMINED AND ONLY THE NONZERO ENTRIES IN THE		!SIM01070
!              UPPER TRIANGLE OF M ARE BEING STORED).					!SIM01080
!           4  SAME AS 2 BUT PUT DIAGONAL ENTRIES AT START OF EACH ROW.	!SIM01090
!           5  SAME AS 3 BUT PUT DIAGONAL ENTRIES AT START OF EACH ROW.	!SIM01100
!                                                                       !SIM01110
!    FLAG - INTEGER ERROR FLAG; VALUES AND THEIR MEANINGS ARE:			!SIM01120
!                                                                       !SIM01130
!           0		NO ERRORS DETECTED.									!SIM01140
!           9N+K	INSUFFICIENT STORAGE IN MD.							!SIM01150
!           10N+1	INSUFFICIENT STORAGE IN ODRV.						!SIM01160
!           11N+1	ILLEGAL PATH SPECIFICATION.							!SIM01170
!                                                                       !SIM01180
!ATTENTION---------------------------------------------------------		!SIM01190
!CONVERSION FROM REAL TO DOUBLE PRECISION: THE REAL DECLARATIONS IN 	!SIM01200
!ODRV AND SRO WERE CHANGED TO DOUBLE PRECISION.							!SIM01210
!------------------------------------------------------------------		!SIM01220
!																		!SIM01230
!VARIABLE DECLARATION***************************************************!SIM01240
 INTEGER(K4B), INTENT(IN):: N, NSP, PATH                                !SIM01250
 INTEGER(K4B), INTENT(INOUT)::  IA(N+1),JA(*),P(N),IP(N),ISP(NSP),FLAG  !SIM01260
 REAL(DP), INTENT(INOUT):: A(*)                                         !SIM01270
!REAL, INTENT(INOUT)::  A(*)											!SIM01280
 INTEGER(K4B):: V, L, HEAD, TMP, Q										!SIM01290
 LOGICAL  DFLAG															!SIM01300
!***********************************************************************!SIM01310
!																		!SIM01320
!INITIALIZE ERROR FLAG AND VALIDATE PATH SPECIFICATION					!SIM01330
 FLAG = 0																!SIM01340
 IF (PATH.LT.1 .OR. 5.LT.PATH) GOTO 111									!SIM01350
!                                                                       !SIM01360
!ALLOCATE STORAGE AND FIND MINIMUM DEGREE ORDERING						!SIM01370
 IF ((PATH-1) * (PATH-2) * (PATH-4) .NE. 0) GOTO 1						!SIM01380
	MAX = (NSP-N)/2														!SIM01390
    V    = 1															!SIM01400
    L    = V     +  MAX													!SIM01410
    HEAD = L     +  MAX													!SIM01420
    NEXT = HEAD  +  N													!SIM01430
    IF (MAX.LT.N) GOTO 110												!SIM01440
!                                                                       !SIM01450
    CALL MD (N,IA,JA,MAX,ISP(V),ISP(L),ISP(HEAD),P,IP,ISP(V),FLAG)		!SIM01460
    IF (FLAG.NE.0) GOTO 100												!SIM01470
!                                                                       !SIM01480
!ALLOCATE STORAGE AND SYMMETRICALLY REORDER MATRIX						!SIM01490
 1  IF ((PATH-2) * (PATH-3) * (PATH-4) * (PATH-5) .NE. 0) GOTO 2		!SIM01500
		TMP = (NSP+1) - N												!SIM01510
		Q   = TMP - (IA(N+1)-1)											!SIM01520
		IF (Q.LT.1)  GOTO 110											!SIM01530
!                                                                       !SIM01540
		DFLAG = PATH.EQ.4 .OR. PATH.EQ.5								!SIM01550
		CALL SRO (N, IP, IA, JA, A, ISP(TMP), ISP(Q), DFLAG)			!SIM01560
!																	    !SIM01570
 2  RETURN																!SIM01580
!				                                                        !SIM01590
!ERROR SPECIFICATION:                                                   !SIM01600
!-------------------													!SIM01610
 100  RETURN  !ERROR DETECTED IN MD									    !SIM01620
!																		!SIM01630
 110  FLAG = 10*N + 1 !INSUFFICIENT STORAGE								!SIM01640
	  RETURN															!SIM01650
!																		!SIM01660
 111  FLAG = 11*N + 1 !ILLEGAL PATH SPECIFIED							!SIM01670
!			                                                            !SIM01680
 END SUBROUTINE ODRV                                                    !SIM01690
!***																	!SIM01700
!																		!SIM01710
!***********************************************************************!SIM01720
 SUBROUTINE  MD (N,IA,JA,MAX,V,L,HEAD,LAST,NEXT,MARK,FLAG)				!SIM01730
!***********************************************************************!SIM01740
!MINIMUM DEGREE ALGORITHM (BASED ON ELEMENT MODEL).						!SIM01750
!																		!SIM01760
!-----------                                                            !SIM01770
!DESCRIPTION															!SIM01780
!-----------                                                            !SIM01790
!MD FINDS A MINIMUM DEGREE ORDERING OF THE ROWS AND COLUMNS OF A		!SIM01800
!SYMMETRIC MATRIX M STORED IN (IA,JA,A) FORMAT.							!SIM01810
!                                                                       !SIM01820
!---------------------                                                  !SIM01830                                                                       !SIM01820
!ADDITIONAL PARAMETERS													!SIM01840
!---------------------                                                  !SIM01850
!    MAX  - DECLARED DIMENSION OF THE WORK VECTORS V AND L; MAX MUST	!SIM01860
!           BE AT LEAST N+2K, WHERE K IS THE NUMBER OF NONZEROES IN     !SIM01870
!           THE STRICT UPPER TRIANGLE OF M					            !SIM01880
!                                                                       !SIM01890
!    V    - WORK VECTOR. DIMENSION = MAX								!SIM01900
!                                                                       !SIM01910
!    L    - WORK VECTOR. DIMENSION = MAX								!SIM01920
!                                                                       !SIM01930
!    HEAD - WORK VECTOR. DIMENSION = N  								!SIM01940
!                                                                       !SIM01950
!    LAST - INTEGER VECTOR USED TO RETURN THE PERMUTATION OF THE ROWS	!SIM01960
!           AND COLUMNS OF M CORRESPONDING TO THE MINIMUM DEGREE ORD-   !SIM01970
!           ERING. DIMENSION = N										!SIM01980
!                                                                       !SIM01990
!    NEXT - INTEGER VECTOR USED TO RETURN THE INVERSE OF THE PERMUTA-	!SIM02000
!           TION RETURNED IN LAST. DIMENSION = N						!SIM02010
!                                                                       !SIM02020
!    MARK - INTEGER WORK VECTOR (MAY BE THE SAME AS V). DIMENSION = N	!SIM02030
!																		!SIM02040
!    FLAG - INTEGER ERROR FLAG;  VALUES AND THEIR MEANINGS ARE:	        !SIM02050
!                                                                       !SIM02060
!           0      NO ERRORS DETECTED.	                                !SIM02070
!           11N+1  INSUFFICIENT STORAGE IN MD.	                        !SIM02080
!                                                                       !SIM02090
!----------------------------------                                     !SIM02100
!DEFINITIONS OF INTERNAL PARAMETERS										!SIM02110
!----------------------------------                                     !SIM02120
!																		!SIM02130
! V(S)     | VALUE FIELD OF LIST ENTRY									!SIM02140
! ---------+---------------------------------------------------------	!SIM02150
! L(S)     | LINK FIELD OF LIST ENTRY  (0 => END OF LIST)				!SIM02160
! ---------+---------------------------------------------------------	!SIM02170
! L(VI)    | POINTER TO ELEMENT LIST OF UNELIMINATED VERTEX VI			!SIM02180
! ---------+---------------------------------------------------------	!SIM02190
! L(EJ)    | POINTER TO BOUNDARY LIST OF ACTIVE ELEMENT EJ				!SIM02200
! ---------+---------------------------------------------------------	!SIM02210
! HEAD(D)  | VJ => VJ HEAD OF D-LIST D									!SIM02220
!          |  0 => NO VERTEX IN D-LIST D								!SIM02230
!																		!SIM02240
!																		!SIM02250
!          |                  VI UNELIMINATED VERTEX					!SIM02260
!          |          VI IN EK           |       VI NOT IN EK			!SIM02270
! ---------+-----------------------------+---------------------------	!SIM02280
! NEXT(VI) | UNDEFINED BUT NONNEGATIVE   | VJ => VJ NEXT IN D-LIST		!SIM02290
!          |                             |  0 => VI TAIL OF D-LIST		!SIM02300
! ---------+-----------------------------+---------------------------	!SIM02310
! LAST(VI) | (NOT SET UNTIL MDP)         | -D => VI HEAD OF D-LIST D	!SIM02320
!          |-VK => COMPUTE DEGREE        | VJ => VJ LAST IN D-LIST		!SIM02330
!          | EJ => VI PROTOTYPE OF EJ    |  0 => VI NOT IN ANY D-LIST	!SIM02340
!          |  0 => DO NOT COMPUTE DEGREE |								!SIM02350
! ---------+-----------------------------+---------------------------	!SIM02360
! MARK(VI) | MARK(VK)                    | NONNEGATIVE TAG < MARK(VK)	!SIM02370
!																		!SIM02380
!																		!SIM02390
!          |                   VI ELIMINATED VERTEX						!SIM02400
!          |      EI ACTIVE ELEMENT      |           OTHERWISE			!SIM02410
! ---------+-----------------------------+---------------------------	!SIM02420
! NEXT(VI) | -J => VI WAS J-TH VERTEX    | -J => VI WAS J-TH VERTEX		!SIM02430
!          |       TO BE ELIMINATED      |       TO BE ELIMINATED		!SIM02440
! ---------+-----------------------------+---------------------------	!SIM02450
! LAST(VI) |  M => SIZE OF EI = M        | UNDEFINED					!SIM02460
! ---------+-----------------------------+---------------------------	!SIM02470
! MARK(VI) | -M => OVERLAP COUNT OF EI   | UNDEFINED					!SIM02480
!          |       WITH EK = M           |								!SIM02490
!          | OTHERWISE NONNEGATIVE TAG   |								!SIM02500
!          |       < MARK(VK)            |								!SIM02510
!																		!SIM02520
!										                                !SIM02530
!VARIABLE DECLARATION***************************************************!SIM02540
 INTEGER(K4B):: IA(N+1),JA(*),V(MAX),L(MAX),HEAD(N),LAST(N),NEXT(N) 	!SIM02550
 INTEGER(K4B):: MARK(N), FLAG, TAG, DMIN, VK, EK, TAIL					!SIM02560
 EQUIVALENCE (VK,EK)													!SIM02570
!***********************************************************************!SIM02580
!                                                                       !SIM02590
!INITIALIZATION															!SIM02600
 TAG = 0																!SIM02610
 CALL MDI (N, IA, JA, MAX, V, L, HEAD, LAST, NEXT, MARK, TAG, FLAG)		!SIM02620
 IF (FLAG.NE.0) RETURN													!SIM02630
!                                                                       !SIM02640
 K = 0																	!SIM02650
 DMIN = 1																!SIM02660
!                                                                       !SIM02670
!	WHILE K < N DO														!SIM02680
 1  IF (K.GE.N) GOTO 4													!SIM02690
!                                                                       !SIM02700
!	SEARCH FOR VERTEX OF MINIMUM DEGREE									!SIM02710
 2  IF (HEAD(DMIN).GT.0) GOTO 3											!SIM02720
    DMIN = DMIN + 1														!SIM02730
    GOTO 2																!SIM02740
!                                                                       !SIM02750
!	REMOVE VERTEX VK OF MINIMUM DEGREE FROM DEGREE LIST					!SIM02760
 3  VK = HEAD(DMIN)														!SIM02770
    HEAD(DMIN) = NEXT(VK)												!SIM02780
    IF (HEAD(DMIN).GT.0)  LAST(HEAD(DMIN)) = -DMIN						!SIM02790
!                                                                       !SIM02800
!	NUMBER VERTEX VK, ADJUST TAG, AND TAG VK							!SIM02810
    K = K+1																!SIM02820
    NEXT(VK) = -K														!SIM02830
    LAST(EK) = DMIN - 1													!SIM02840
    TAG = TAG + LAST(EK)												!SIM02850
    MARK(VK) = TAG														!SIM02860
!                                                                       !SIM02870
!	FORM ELEMENT EK FROM UNELIMINATED NEIGHBORS OF VK					!SIM02880
    CALL MDM (VK, TAIL, V, L, LAST, NEXT, MARK)							!SIM02890
!                                                                       !SIM02900
!	PURGE INACTIVE ELEMENTS AND DO MASS ELIMINATION					    !SIM02910
    CALL MDP (K, EK, TAIL, V, L, HEAD, LAST, NEXT, MARK)				!SIM02920
!																		!SIM02930
!	UPDATE DEGREES OF UNELIMINATED VERTICES IN EK						!SIM02940
    CALL MDU (EK, DMIN, V, L, HEAD, LAST, NEXT, MARK)                   !SIM02950
!																		!SIM02960
    GOTO 1																!SIM02970
!                                                                       !SIM02980
!	GENERATE INVERSE PERMUTATION FROM PERMUTATION						!SIM02990
 4  DO 5 K=1,N															!SIM03000
	NEXT(K) = -NEXT(K)													!SIM03010
!                                                                       !SIM03020
 5  LAST(NEXT(K)) = K													!SIM03030
!																		!SIM03040
 END SUBROUTINE MD                                                      !SIM03050
!                                                                       !SIM03060
!***********************************************************************!SIM03070
 SUBROUTINE  MDI (N,IA,JA,MAX,V,L,HEAD,LAST,NEXT,MARK,TAG,FLAG)			!SIM03080
!***********************************************************************!SIM03090
!INITIALIZATION.														!SIM03100
!																		!SIM03110
!VARIABLE DECLARATION***************************************************!SIM03120
 INTEGER(K4B):: IA(N+1), JA(*), V(MAX),L(MAX),HEAD(N),LAST(N), NEXT(N)	!SIM03130
 INTEGER(K4B):: MARK(N), TAG, FLAG, SFS, VI, DVI, VJ					!SIM03140
!***********************************************************************!SIM03150
!	                                                                    !SIM03160
!   INITIALIZE DEGREES, ELEMENT LISTS, AND DEGREE LISTS					!SIM03170
    DO 1 VI=1,N															!SIM03180
		MARK(VI) = 1                                                    !SIM03190
        L(VI) = 0                                                       !SIM03200
 1	HEAD(VI) = 0														!SIM03210
    SFS = N+1															!SIM03220
!                                                                       !SIM03230
!   CREATE NONZERO STRUCTURE FOR EACH NONZERO ENTRY A(VI,VJ) IN STRICT	!SIM03240
!   UPPER TRIANGLE														!SIM03250
    DO 3 VI=1,N															!SIM03260
		JMIN = IA(VI)													!SIM03270
		JMAX = IA(VI+1) - 1												!SIM03280
		IF (JMIN.GT.JMAX) GOTO 3										!SIM03290
		DO 2 J=JMIN,JMAX												!SIM03300
			VJ = JA(J)                                                  !SIM03310
			IF (VI.GE.VJ) GOTO 2										!SIM03320
			IF (SFS.GE.MAX) GOTO 101									!SIM03330
!                                                                       !SIM03340
!			ENTER VJ IN ELEMENT LIST FOR VI                             !SIM03350
			MARK(VI) = MARK(VI) + 1										!SIM03360
			V(SFS) = VJ													!SIM03370
			L(SFS) = L(VI)												!SIM03380
			L(VI) = SFS													!SIM03390
			SFS = SFS+1													!SIM03400
!                                                                       !SIM03410
!			ENTER VI IN ELEMENT LIST FOR VJ                             !SIM03420
			MARK(VJ) = MARK(VJ) + 1										!SIM03430
			V(SFS) = VI													!SIM03440
			L(SFS) = L(VJ)												!SIM03450
			L(VJ) = SFS													!SIM03460
			SFS = SFS+1													!SIM03470
!																		!SIM03480
 2  CONTINUE															!SIM03490
 3  CONTINUE															!SIM03500
!                                                                       !SIM03510
!   CREATE DEGREE LISTS AND INITIALIZE MARK VECTOR						!SIM03520
    DO 4 VI=1,N															!SIM03530
		DVI = MARK(VI)                                                  !SIM03540
        NEXT(VI) = HEAD(DVI)                                            !SIM03550
        HEAD(DVI) = VI                                                  !SIM03560
        LAST(VI) = -DVI                                                 !SIM03570
        IF (NEXT(VI).GT.0) LAST(NEXT(VI)) = VI							!SIM03580
 4  MARK(VI) = TAG														!SIM03590
    RETURN                                                              !SIM03600
!                                                                       !SIM03610
!ERROR: INSUFFICIENT STORAGE											!SIM03620
 101	FLAG = 9*N + VI                                                 !SIM03630
!			                                                            !SIM03640
 END SUBROUTINE MDI                                                     !SIM03650
!																		!SIM03660
!***********************************************************************!SIM03670
 SUBROUTINE MDM (VK, TAIL, V, L, LAST, NEXT, MARK)						!SIM03680
!***********************************************************************!SIM03690
!FORM ELEMENT FROM UNELIMINATED NEIGHBORS OF VK.						!SIM03700
!																		!SIM03710
!VARIABLE DECLARATION***************************************************!SIM03720
 INTEGER(K4B):: VK, TAIL, V(*), L(*), LAST(*), NEXT(*), MARK(*)			!SIM03730
 INTEGER(K4B):: TAG, S, LS, VS, ES, B, LB, VB, BLP, BLPMAX				!SIM03740
 EQUIVALENCE (VS, ES)													!SIM03750
!***********************************************************************!SIM03760
!INITIALIZE TAG AND LIST OF UNELIMINATED NEIGHBORS						!SIM03770
 TAG = MARK(VK)															!SIM03780
 TAIL = VK																!SIM03790
!                                                                       !SIM03800
!	FOR EACH VERTEX/ELEMENT VS/ES IN ELEMENT LIST OF VK					!SIM03810
	LS = L(VK)															!SIM03820
 1  S = LS															    !SIM03830
    IF (S.EQ.0) GOTO 5													!SIM03840
	LS = L(S)															!SIM03850
    VS = V(S)															!SIM03860
    IF (NEXT(VS).LT.0) GOTO 2											!SIM03870
!                                                                       !SIM03880
!	IF VS IS UNELIMINATED VERTEX, THEN TAG AND APPEND TO LIST OF		!SIM03890
!	UNELIMINATED NEIGHBORS												!SIM03900
    MARK(VS) = TAG														!SIM03910
    L(TAIL) = S															!SIM03920
    TAIL = S															!SIM03930
    GOTO 4																!SIM03940
!                                                                       !SIM03950
!	IF ES IS ACTIVE ELEMENT, THEN FOR EACH VERTEX VB IN BOUNDARY        !SIM03960
!	LIST OF ELEMENT ES													!SIM03970
 2	LB = L(ES)															!SIM03980
    BLPMAX = LAST(ES)													!SIM04990
    DO 3 BLP=1,BLPMAX													!SIM04000
		B = LB															!SIM04010
        LB = L(B)														!SIM04020
        VB = V(B)														!SIM04030
!                                                                       !SIM04040
!		IF VB IS UNTAGGED VERTEX, THEN TAG AND APPEND TO LIST OF		!SIM04050
!		UNELIMINATED NEIGHBORS											!SIM04060
        IF (MARK(VB).GE.TAG) GOTO 3										!SIM04070
        MARK(VB) = TAG													!SIM04080
        L(TAIL) = B														!SIM04090
        TAIL = B														!SIM04100
 3  CONTINUE															!SIM04110
!                                                                       !SIM04120
!	MARK ES INACTIVE													!SIM04130
    MARK(ES) = TAG														!SIM04140
!                                                                       !SIM04150
 4	GOTO 1																!SIM04160
!                                                                       !SIM04170
!	TERMINATE LIST OF UNELIMINATED NEIGHBORS							!SIM04180
 5	L(TAIL) = 0															!SIM04190
!																		!SIM04200
 END SUBROUTINE MDM                                                     !SIM04210
!                                                                       !SIM04220
!***********************************************************************!SIM04230
 SUBROUTINE MDP (K, EK, TAIL, V, L, HEAD, LAST, NEXT, MARK)				!SIM04240
!***********************************************************************!SIM04250
!PURGE INACTIVE ELEMENTS AND DO MASS ELIMINATION						!SIM04260
!											                            !SIM04270
!VARIABLE DECLARATION***************************************************!SIM04280
 INTEGER(K4B):: EK, TAIL, V(*), L(*), HEAD(*), LAST(*), NEXT(*)			!SIM04290
 INTEGER(K4B):: MARK(*), TAG,FREE,LI,VI,LVI,EVI, S, LS, ES, ILP, ILPMAX !SIM04300
!***********************************************************************!SIM04310
!																		!SIM04320
!	INITIALIZE TAG														!SIM04330
	TAG = MARK(EK)														!SIM04340
!                                                                       !SIM04350
!	FOR EACH VERTEX VI IN EK											!SIM04360
	LI = EK																!SIM04370
	ILPMAX = LAST(EK)													!SIM04380
	IF (ILPMAX.LE.0) GOTO 12											!SIM04390
	DO 11 ILP=1,ILPMAX													!SIM04400
		 I = LI															!SIM04410
		LI = L(I)														!SIM04420
		VI = V(LI)														!SIM04430
!                                                                       !SIM04440
!		REMOVE VI FROM DEGREE LIST										!SIM04450
		IF (LAST(VI).EQ.0) GOTO 3										!SIM04460
		IF (LAST(VI).GT.0) GOTO 1										!SIM04470
		HEAD(-LAST(VI)) = NEXT(VI)										!SIM04480
		GOTO 2															!SIM04490
 1		NEXT(LAST(VI)) = NEXT(VI)										!SIM04500
 2		IF (NEXT(VI).GT.0) LAST(NEXT(VI)) = LAST(VI)					!SIM04510
!                                                                       !SIM04520
!		REMOVE INACTIVE ITEMS FROM ELEMENT LIST OF VI					!SIM04530
 3		LS = VI															!SIM04540
 4		S = LS															!SIM04550
		LS = L(S)														!SIM04560
		IF (LS.EQ.0) GOTO 6												!SIM04570
		ES = V(LS)														!SIM04580
		IF (MARK(ES).LT.TAG) GOTO 5										!SIM04590
		FREE = LS														!SIM04600
		L(S) = L(LS)													!SIM04610
		LS = S															!SIM04620
 5		GOTO 4															!SIM04630
!                                                                       !SIM04640
!		IF VI IS INTERIOR VERTEX, THEN REMOVE FROM LIST AND ELIMINATE	!SIM04650
 6		LVI = L(VI)														!SIM04660
		IF (LVI.NE.0) GOTO 7											!SIM04670
		L(I) = L(LI)													!SIM04680
		LI = I															!SIM04690
!                                                                       !SIM04700
		K = K+1															!SIM04710
		NEXT(VI) = -K													!SIM04720
		LAST(EK) = LAST(EK) - 1											!SIM04730
		GOTO 11															!SIM04740
!                                                                       !SIM04750
!		ELSE CLASSIFY VERTEX VI                                         !SIM04760
 7		IF (L(LVI).NE.0) GOTO 9											!SIM04770
		EVI = V(LVI)													!SIM04780
		IF (NEXT(EVI).GE.0) GOTO 9										!SIM04790
		IF (MARK(EVI).LT.0) GOTO 8										!SIM04800
!                                                                       !SIM04810
!		IF VI IS PROTOTYPE VERTEX, THEN MARK AS SUCH, INITIALIZE		!SIM04820
!		OVERLAP COUNT FOR CORRESPONDING ELEMENT, AND MOVE VI TO  		!SIM04830
!		END OF BOUNDARY LIST											!SIM04840
		LAST(VI) = EVI													!SIM04850
		MARK(EVI) = -1													!SIM04860
		L(TAIL) = LI													!SIM04870
		TAIL = LI														!SIM04880
		L(I) = L(LI)													!SIM04890
		LI = I															!SIM04900
		GOTO 10															!SIM04910
!                                                                       !SIM04920
!		ELSE IF VI IS DUPLICATE VERTEX, THEN MARK AS SUCH AND ADJUST 	!SIM04930
!		OVERLAP COUNT FOR CORRESPONDING ELEMENT							!SIM04940
 8		LAST(VI) = 0													!SIM04950
		MARK(EVI) = MARK(EVI) - 1										!SIM04960
		GOTO 10															!SIM04970
!                                                                       !SIM04980
!		ELSE MARK VI TO COMPUTE DEGREE									!SIM04990
 9		LAST(VI) = -EK													!SIM05000
!		                                                                !SIM05010
!		INSERT EK IN ELEMENT LIST OF VI									!SIM05020
 10		V(FREE) = EK													!SIM05030
		L(FREE) = L(VI)													!SIM05040
		L(VI) = FREE													!SIM05050
 11 CONTINUE															!SIM05060
!                                                                       !SIM05070
!	TERMINATE BOUNDARY LIST												!SIM05080
 12 L(TAIL) = 0															!SIM05090
!                                                                       !SIM05100
 END SUBROUTINE MDP                                                     !SIM05110
!                                                                       !SIM05120
!***********************************************************************!SIM05130
 SUBROUTINE MDU (EK, DMIN, V, L, HEAD, LAST, NEXT, MARK)                !SIM05140
!***********************************************************************!SIM05150
!UPDATE DEGREES OF UNELIMINATED VERTICES IN EK							!SIM05160
!																		!SIM05170
!VARIABLE DECLARATION***************************************************!SIM05180
 INTEGER(K4B):: EK, DMIN, V(*), L(*), HEAD(*), LAST(*),NEXT(*), MARK(*)	!SIM05190
 INTEGER(K4B):: TAG, VI, EVI,DVI,S,VS,ES,B, VB, ILP, ILPMAX, BLP,BLPMAX	!SIM05200
 EQUIVALENCE (VS, ES)													!SIM05210
!***********************************************************************!SIM05220
!	INITIALIZE TAG														!SIM05230
	TAG = MARK(EK) - LAST(EK)											!SIM05240
!                                                                       !SIM05250
!	FOR EACH VERTEX VI IN EK											!SIM05260
	I = EK																!SIM05270
    ILPMAX = LAST(EK)													!SIM05280
    IF (ILPMAX.LE.0) GOTO 11											!SIM05290
    DO 10 ILP=1,ILPMAX													!SIM05300
		I = L(I)														!SIM05310
		VI = V(I)														!SIM05320
		IF (LAST(VI)) 1, 10, 8											!SIM05330
!                                                                       !SIM05340
!		IF VI NEITHER PROTOTYPE NOR DUPLICATE VERTEX, THEN MERGE		!SIM05350
!		ELEMENTS TO COMPUTE DEGREE                                      !SIM05360
 1		TAG = TAG + 1													!SIM05370
        DVI = LAST(EK)													!SIM05380
!                                                                       !SIM05390
!		FOR EACH VERTEX/ELEMENT VS/ES IN ELEMENT LIST OF VI				!SIM05400
        S = L(VI)														!SIM05410
 2      S = L(S)														!SIM05420
        IF (S.EQ.0) GOTO 9												!SIM05430
        VS = V(S)														!SIM05440
        IF (NEXT(VS).LT.0) GOTO 3										!SIM05450
!                                                                       !SIM05460
!		IF VS IS UNELIMINATED VERTEX, THEN TAG AND ADJUST DEGREE		!SIM05470
        MARK(VS) = TAG													!SIM05480
        DVI = DVI + 1													!SIM05490
        GOTO 5															!SIM05500
!                                                                       !SIM05510
!		IF ES IS ACTIVE ELEMENT, THEN EXPAND CHECK FOR OUTMATCHED		!SIM05520
!		VERTEX															!SIM05530
 3      IF (MARK(ES).LT.0) GOTO 6										!SIM05540
!                                                                       !SIM05550
!		FOR EACH VERTEX VB IN ES										!SIM05560
        B = ES															!SIM05570
        BLPMAX = LAST(ES)												!SIM05580
        DO 4 BLP=1,BLPMAX												!SIM05590
			B = L(B)                                                    !SIM05600
			VB = V(B)                                                   !SIM05610
!                                                                       !SIM05620
!			IF VB IS UNTAGGED, THEN TAG AND ADJUST DEGREE				!SIM05630
			IF (MARK(VB).GE.TAG) GOTO 4									!SIM05640
            MARK(VB) = TAG												!SIM05650
            DVI = DVI + 1												!SIM05660
 4		CONTINUE														!SIM05670
 5		GOTO 2															!SIM05680
!                                                                       !SIM05690
!		ELSE IF VI IS OUTMATCHED VERTEX, THEN ADJUST OVERLAPS BUT DO	!SIM05700
!		NOT COMPUTE DEGREE                                              !SIM05710
 6      LAST(VI) = 0													!SIM05720
        MARK(ES) = MARK(ES) - 1											!SIM05730
 7      S = L(S)														!SIM05740
        IF (S.EQ.0)  GOTO 10											!SIM05750
        ES = V(S)														!SIM05760
        IF (MARK(ES).LT.0)  MARK(ES) = MARK(ES) - 1						!SIM05770
        GOTO 7															!SIM05780
!                                                                       !SIM05790
!		ELSE IF VI IS PROTOTYPE VERTEX, THEN CALCULATE DEGREE BY        !SIM05800
!		INCLUSION/EXCLUSION AND RESET OVERLAP COUNT                     !SIM05810
 8      EVI = LAST(VI)													!SIM05820
        DVI = LAST(EK) + LAST(EVI) + MARK(EVI)							!SIM05830
        MARK(EVI) = 0													!SIM05840
!                                                                       !SIM05850
!		INSERT VI IN APPROPRIATE DEGREE LIST                            !SIM05860
 9      NEXT(VI) = HEAD(DVI)											!SIM05870
        HEAD(DVI) = VI                                                  !SIM05880
        LAST(VI) = -DVI                                                 !SIM05890
        IF (NEXT(VI).GT.0)  LAST(NEXT(VI)) = VI                         !SIM05900
        IF (DVI.LT.DMIN)  DMIN = DVI                                    !SIM05910
!                                                                       !SIM05920
 10	CONTINUE															!SIM05930
!                                                                       !SIM05940
 11 RETURN																!SIM05950
!                                                                       !SIM05960
 END SUBROUTINE MDU                                                     !SIM05970
!																		!SIM05980
!***********************************************************************!SIM05990
 SUBROUTINE SRO (N, IP, IA, JA, A, Q, R, DFLAG)							!SIM06000
!***********************************************************************!SIM06010
!SYMMETRIC REORDERING OF SPARSE SYMMETRIC MATRIX.						!SIM06020
!																		!SIM06030
!-----------															!SIM06040
!DESCRIPTION															!SIM06050
!-----------                                                            !SIM06060
!                                                                       !SIM06070
!THE NONZERO ENTRIES OF THE MATRIX M ARE ASSUMED TO BE STORED SYMME-	!SIM06080
!TRICALLY IN (IA,JA,A) FORMAT (I.E., NOT BOTH M(I,J) AND M(J,I) ARE		!SIM06090
!STORED IF I NE J).														!SIM06100
!                                                                       !SIM06110
!SRO DOES NOT REARRANGE THE ORDER OF THE ROWS, BUT DOES MOVE NONZEROES  !SIM06120
!FROM ONE ROW TO ANOTHER TO ENSURE THAT IF M(I,J) WILL BE IN THE UPPER	!SIM06130
!TRIANGLE OF M WITH RESPECT TO THE NEW ORDERING, THEN M(I,J) IS STORED  !SIM06140
!IN ROW I (AND THUS M(J,I) IS NOT STORED); WHEREAS IF M(I,J) WILL BE IN	!SIM06150
!THE STRICT LOWER TRIANGLE OF M, THEN M(J,I) IS STORED IN ROW J (AND	!SIM06160
!THUS M(I,J) IS NOT STORED).											!SIM06170
!                                                                       !SIM06180
!---------------------                                                  !SIM06190
!ADDITIONAL PARAMETERS													!SIM06200
!---------------------                                                  !SIM06210
!    Q     - INTEGER WORK VECTOR. DIMENSION = N							!SIM06220
!                                                                       !SIM06230
!    R     - INTEGER WORK VECTOR. DIMENSION = NUMBER OF NONZERO ENTRIES !SIM06240
!            IN THE UPPER TRIANGLE OF M									!SIM06250
!                                                                       !SIM06260
!    DFLAG - LOGICAL VARIABLE; IF DFLAG = .TRUE., THEN STORE NONZERO	!SIM06270
!            DIAGONAL ELEMENTS AT THE BEGINNING OF THE ROW              !SIM06280
!																		!SIM06290
!VARIABLE DECLARATION***************************************************!SIM06300
 INTEGER(K4B):: IP(N), IA(N+1), JA(*),  Q(N), R(*)						!SIM06310
!REAL:: A(*), AK														!SIM06320
 REAL(DP):: A(*), AK										        	!SIM06330
 LOGICAL:: DFLAG														!SIM06340
!***********************************************************************!SIM06350
!                                                                       !SIM06360
!PHASE 1: FIND ROW IN WHICH TO STORE EACH NONZERO.						!SIM06370
!																	    !SIM06380
!	INITIALIZE COUNT OF NONZEROES TO BE STORED IN EACH ROW				!SIM06390
    DO 1 I=1,N															!SIM06400
 1	Q(I) = 0															!SIM06410
!                                                                       !SIM06420
!	FOR EACH NONZERO ELEMENT A(J)										!SIM06430
    DO 3 I=1,N															!SIM06440
	    JMIN = IA(I)                                                    !SIM06450
        JMAX = IA(I+1) - 1                                              !SIM06460
        IF (JMIN.GT.JMAX) GOTO 3										!SIM06470
        DO 2 J=JMIN,JMAX                                                !SIM06480
!                                                                       !SIM06490
!			FIND ROW (=R(J)) AND COLUMN (=JA(J)) IN WHICH TO STORE A(J)	!SIM06500
			K = JA(J)                                                   !SIM06510
			IF (IP(K).LT.IP(I))  JA(J) = I                              !SIM06520
			IF (IP(K).GE.IP(I))  K = I                                  !SIM06530
			R(J) = K                                                    !SIM06540
!                                                                       !SIM06550
!			INCREMENT COUNT OF NONZEROES (=Q(R(J)) IN THAT ROW			!SIM06560
 2			Q(K) = Q(K) + 1                                             !SIM06570
 3  CONTINUE															!SIM06580
!                                                                       !SIM06590
!PHASE 2: FIND NEW IA AND PERMUTATION TO APPLY TO (JA,A)				!SIM06600
!                                                                       !SIM06610
!	DETERMINE POINTERS TO DELIMIT ROWS IN PERMUTED (JA,A)				!SIM06620
    DO 4 I=1,N															!SIM06630
		IA(I+1) = IA(I) + Q(I)                                          !SIM06640
4		Q(I) = IA(I+1)													!SIM06650
!                                                                       !SIM06660
!	DETERMINE WHERE EACH (JA(J),A(J)) IS STORED IN PERMUTED (JA,A)		!SIM06670
!	FOR EACH NONZERO ELEMENT (IN REVERSE ORDER)							!SIM06680
    ILAST = 0															!SIM06690
    JMIN = IA(1)														!SIM06700
    JMAX = IA(N+1) - 1													!SIM06710
    J = JMAX															!SIM06720
    DO 6 JDUMMY=JMIN,JMAX												!SIM06730
        I = R(J)                                                        !SIM06740
        IF (.NOT.DFLAG .OR. JA(J).NE.I .OR. I.EQ.ILAST) GOTO 5			!SIM06750
!                                                                       !SIM06760
!		IF DFLAG, THEN PUT DIAGONAL NONZERO AT BEGINNING OF ROW         !SIM06770
        R(J) = IA(I)													!SIM06780
        ILAST = I														!SIM06790
        GOTO 6															!SIM06800
!                                                                       !SIM06810
!		PUT (OFF-DIAGONAL) NONZERO IN LAST UNUSED LOCATION IN ROW		!SIM06820
 5      Q(I) = Q(I) - 1													!SIM06830
        R(J) = Q(I)														!SIM06840
 6		J = J-1															!SIM06850
!                                                                       !SIM06860
!PHASE 3: PERMUTE (JA,A) TO UPPER TRIANGULAR FORM (WRT NEW ORDERING)	!SIM06870
!																		!SIM06880
	DO 8 J=JMIN,JMAX													!SIM06890
 7		IF (R(J).EQ.J) GOTO 8											!SIM06900
		K = R(J)														!SIM06910
		R(J) = R(K)														!SIM06920
		R(K) = K														!SIM06930
		JAK = JA(K)														!SIM06940
		JA(K) = JA(J)													!SIM06950
		JA(J) = JAK														!SIM06960
		AK = A(K)														!SIM06970
		A(K) = A(J)														!SIM06980
		A(J) = AK														!SIM06990
		GOTO 7															!SIM07000
 8	CONTINUE															!SIM07010
!                                                                       !SIM07020
 END SUBROUTINE SRO                                                     !SIM07030
!																		!SIM07040
!***********************************************************************!SIM07050
 SUBROUTINE SDRV (N,P,IP,IA,JA,A,B,Z,NSP,ISP,RSP,ESP,PATH,FLAG)			!SIM07060
!***********************************************************************!SIM07070
!DRIVER FOR SPARSE SYMMETRIC POSITIVE DEFINITE MATRIX ROUTINES.			!SIM07080
!																		!SIM07090
!-----------												            !SIM07100
!DESCRIPTION		                                                    !SIM07110
!-----------	                                                        !SIM07120
!SDRV SOLVES SPARSE SYMMETRIC POSITIVE DEFINITE SYSTEMS OF LINEAR		!SIM07130
!EQUATIONS. THE SOLUTION PROCESS IS DIVIDED INTO THREE STAGES:			!SIM07140
!                                                                       !SIM07150
!      SSF - THE COEFFICIENT MATRIX M IS FACTORED SYMBOLICALLY TO       !SIM07160
!            DETERMINE WHERE FILLIN WILL OCCUR DURING THE NUMERIC       !SIM07170
!            FACTORIZATION.                                             !SIM07180
!                                                                       !SIM07190
!      SNF - M IS FACTORED NUMERICALLY INTO THE PRODUCT UT-D-U,			!SIM07200
!            WHERE D IS DIAGONAL AND U IS UNIT UPPER TRIANGULAR.        !SIM07210
!                                                                       !SIM07220
!      SNS - THE LINEAR SYSTEM MX = B IS SOLVED USING THE UT-D-U		!SIM07230
!            FACTORIZATION FROM SNF.                                    !SIM07240
!                                                                       !SIM07250
!FOR SEVERAL SYSTEMS WITH THE SAME COEFFICIENT MATRIX, SSF AND SNF		!SIM07260
!NEED BE DONE ONLY ONCE (FOR THE FIRST SYSTEM); THEN SNS IS DONE		!SIM07270
!ONCE FOR EACH ADDITIONAL RIGHT-HAND SIDE. FOR SEVERAL SYSTEMS			!SIM07280
!WHOSE COEFFICIENT MATRICES HAVE THE SAME NONZERO STRUCTURE, SSF		!SIM07290
!NEED BE DONE ONLY ONCE (FOR THE FIRST SYSTEM); THEN SNF AND SNS		!SIM07300
!ARE DONE ONCE FOR EACH ADDITIONAL SYSTEM.								!SIM07310
!                                                                       !SIM07320
!                                                                       !SIM07330
!--------------------------                                             !SIM07340
!STORAGE OF SPARSE MATRICES												!SIM07350
!--------------------------                                             !SIM07360
!THE NONZERO ENTRIES OF THE MATRIX M ARE STORED ROW-BY-ROW IN THE		!SIM07370
!ARRAY A. TO IDENTIFY THE INDIVIDUAL NONZERO ENTRIES IN EACH ROW,		!SIM07380
!WE NEED TO KNOW IN WHICH COLUMN EACH ENTRY LIES. THESE COLUMN IN-		!SIM07390
!DICES ARE STORED IN THE ARRAY JA; I.E., IF A(K) = M(I,J), THEN			!SIM07400
!JA(K) = J. TO IDENTIFY THE INDIVIDUAL ROWS, WE NEED TO KNOW WHE-		!SIM07410
!RE EACH ROW STARTS. THESE ROW POINTERS ARE STORED IN THE ARRAY IA;		!SIM07420
!I.E., IF M(I,J) IS THE FIRST NONZERO ENTRY (STORED) IN THE I-TH		!SIM07430
!ROW AND A(K) = M(I,J), THEN IA(I) = K. MOREOVER, IA(N+1) POINTS		!SIM07440
!TO THE FIRST LOCATION FOLLOWING THE LAST ELEMENT IN THE LAST ROW.		!SIM07450
!THUS, THE NUMBER OF ENTRIES IN THE I-TH ROW IS  IA(I+1) - IA(I),		!SIM07460
!THE NONZERO ENTRIES IN THE I-TH ROW ARE STORED CONSECUTIVELY IN		!SIM07470
!                                                                       !SIM07480
!		     A(IA(I)),  A(IA(I)+1),  ..., A(IA(I+1)-1),                 !SIM07490
!                                                                       !SIM07500
!AND THE CORRESPONDING COLUMN INDICES ARE STORED CONSECUTIVELY IN		!SIM07510
!			                                                            !SIM07520
!			JA(IA(I)), JA(IA(I)+1), ..., JA(IA(I+1)-1).					!SIM07530
!                                                                       !SIM07540
!SINCE THE COEFFICIENT MATRIX IS SYMMETRIC, ONLY THE NONZERO ENTRIES	!SIM07550
!IN THE UPPER TRIANGLE NEED BE STORED, FOR EXAMPLE, THE MATRIX			!SIM07560
!                                                                       !SIM07570
!             ( 1  0  2  3  0 )                                         !SIM07580
!             ( 0  4  0  0  0 )                                         !SIM07590
!         M = ( 2  0  5  6  0 )                                         !SIM07600
!             ( 3  0  6  7  8 )                                         !SIM07610
!             ( 0  0  0  8  9 )                                         !SIM07620
!                                                                       !SIM07630
!COULD BE STORED AS														!SIM07640
!                                                                       !SIM07650
!            | 1  2  3  4  5  6  7  8  9 10 11 12 13                    !SIM07660
!         ---+--------------------------------------                    !SIM07670
!         IA | 1  4  5  8 12 14                                         !SIM07680
!         JA | 1  3  4  2  1  3  4  1  3  4  5  4  5                    !SIM07690
!          A | 1  2  3  4  2  5  6  3  6  7  8  8  9                    !SIM07700
!                                                                       !SIM07710
!OR (SYMMETRICALLY) AS													!SIM07720
!                                                                       !SIM07730
!            | 1  2  3  4  5  6  7  8  9                                !SIM07740
!         ---+--------------------------                                !SIM07750
!         IA | 1  4  5  7  9 10                                         !SIM07760
!         JA | 1  3  4  2  3  4  4  5  5                                !SIM07770
!          A | 1  2  3  4  5  6  7  8  9          .                     !SIM07780
!                                                                       !SIM07790
!																		!SIM07800
!------------------------------------	                                !SIM07810
!REORDERING THE ROWS AND COLUMNS OF M									!SIM07820
!------------------------------------	                                !SIM07830
!A SYMMETRIC PERMUTATION OF THE ROWS AND COLUMNS OF THE COEFFICIENT MA-	!SIM07840
!TRIX M (E.G., WHICH REDUCES FILLIN OR ENHANCES NUMERICAL STABILITY)	!SIM07850
!MUST BE SPECIFIED. THE SOLUTION Z IS RETURNED IN THE ORIGINAL ORDER.	!SIM07860
!																		!SIM07870
!TO SPECIFY THE TRIVIAL ORDERING (I.E., THE IDENTITY PERMUTATION), SET	!SIM07880
!P(I) = IP(I) = I, I=1,...,N. IN THIS CASE, P AND IP CAN BE THE SAME A-	!SIM07890
!RRAY.																	!SIM07900
!																		!SIM07910
!IF A NONTRIVIAL ORDERING (I.E., NOT THE IDENTITY PERMUTATION) IS SPE-	!SIM07920
!CIFIED AND M IS STORED SYMMETRICALLY (I.E., NOT BOTH M(I,J) AND M(J,I)	!SIM07930
!ARE STORED FOR I NE J), THEN SUBROUTINE "ODRV" SHOULD BE CALLED (WITH	!SIM07940
!PATH = 3 OR 5. ADDITIONALY, USE PATH 2 OR 4 IF MINIMUM DEGREE ORDERING !SIM07950
!MUST BE FIND FIRST) TO SYMMETRICALLY REORDER (IA,JA,A) BEFORE CALLING	!SIM07960
!SDRV. THIS IS TO ENSURE THAT IF M(I,J) WILL BE IN THE UPPER TRIANGLE	!SIM07970
!OF M WITH RESPECT TO THE NEW ORDERING, THEN M(I,J) IS STORED IN ROW I	!SIM07980
!(AND THUS M(J,I) IS NOT STORED); WHEREAS IF M(I,J) WILL BE IN THE 		!SIM07990
!STRICT LOWER TRIANGLE OF M, THEN M(J,I) IS STORED IN ROW J (AND THUS	!SIM08000
!M(I,J) IS NOT STORED).													!SIM08010
!                                                                       !SIM08020
!PARAMETERS:	                                                        !SIM08030
!                                                                       !SIM08040
!    N    - NUMBER OF VARIABLES/EQUATIONS                               !SIM08050
!                                                                       !SIM08060
!    P    - INTEGER VECTOR SPECIFYING A PERMUTATION OF THE ROWS AND		!SIM08070
!           COLUMNS OF M; DIMENSION = N									!SIM08080
!                                                                       !SIM08090
!    IP   - INTEGER VECTOR CONTAINING THE INVERSE OF THE PERMUTATION	!SIM08100
!           SPECIFIED IN P; I.E., IP(P(I)) = I, I=1,...,N;				!SIM08110
!           DIMENSION = N                                               !SIM08120
!                                                                       !SIM08130
!    IA   - INTEGER VECTRO CONTAINING POINTERS TO DELIMIT ROWS IN JA	!SIM08140
!           AND A; DIMENSION = N+1										!SIM08150
!                                                                       !SIM08160
!    JA   - INTEGER VECTOR CONTAINING THE COLUMN INDICES CORRESPONDING	!SIM08170
!           TO THE ELEMENTS OF A; DIMENSION = NUMBER OF NONZERO ENTRIES !SIM08180
!           IN M STORED													!SIM08190
!                                                                       !SIM08200
!    A    - REAL VECTOR CONTAINING THE NONZERO ENTRIES IN THE COEFFI-	!SIM08210
!           CIENT MATRIX M, STORED BY ROWS; DIMENSION = NUMBER OF	    !SIM08220
!           NONZERO ENTRIES IN M STORED									!SIM08230
!                                                                       !SIM08240
!    B    - REAL VECTOR CONTAINING THE RIGHT-HAND SIDE B; B AND Z CAN	!SIM08250
!           BE THE SAME ARRAY; DIMENSION = N							!SIM08260
!                                                                       !SIM08270
!    Z    - REAL VECTOR CONTAINING THE SOLUTION X; Z AND B CAN BE THE	!SIM08280
!           SAME ARRAY; DIMENSION = N									!SIM08290
!                                                                       !SIM08300
!    NSP  - DECLARED DIMENSION OF THE VECTORS ISP AND RSP; NSP MUST BE  !SIM08310
!           (SUBSTANTIALLY) LARGER THAN 3N+2K, WHERE K = NUMBER OF NON-	!SIM08320
!           ZERO ENTRIES IN THE UPPER TRIANGLE OF M						!SIM08330
!                                                                       !SIM08340
!    ISP  - INTEGER VECTOR USED FOR WORKING STORAGE; ISP AND RSP SHOULD !SIM08350
!           BE EQUIVALENCED; DIMENSION = NSP				            !SIM08360
!                                                                       !SIM08370
!    RSP  - REAL VECTOR USED FOR WORKING STORAGE; RSP AND ISP SHOULD BE !SIM08380
!           EQUIVALENCED; DIMENSION = NSP								!SIM08390
!                                                                       !SIM08400
!    ESP  - INTEGER VARIABLE; IF SUFFICIENT STORAGE WAS AVAILABLE TO	!SIM08410
!           PERFORM THE SYMBOLIC FACTORIZATION (SSF), THEN ESP IS SET	!SIM08420
!           TO THE AMOUNT OF EXCESS STORAGE PROVIDED (NEGATIVE IF IN-   !SIM08430
!           SUFFICIENT STORAGE WAS AVAILABLE TO PERFORM THE NUMERIC		!SIM08440
!           FACTORIZATION (SNF))                                        !SIM08450
!                                                                       !SIM08460
!    PATH - INTEGER PATH SPECIFICATION; VALUES AND THEIR MEANINGS ARE:	!SIM08470
!             1  PERFORM SSF, SNF, AND SNS.                             !SIM08480
!             2  PERFORM SNF AND SNS (ISP/RSP IS ASSUMED TO HAVE BEEN   !SIM08490
!                SET UP IN AN EARLIER CALL TO SDRV (FOR SSF)).			!SIM08500
!             3  PERFORM SNS ONLY (ISP/RSP IS ASSUMED TO HAVE BEEN SET  !SIM08510
!                UP IN AN EARLIER CALL TO SDRV (FOR SSF AND SNF)).	    !SIM08520
!             4  PERFORM SSF.                                           !SIM08530
!             5  PERFORM SSF AND SNF.                                   !SIM08540
!             6  PERFORM SNF ONLY (ISP/RSP IS ASSUMED TO HAVE BEEN SET  !SIM08550
!                UP IN AN EARLIER CALL TO SDRV (FOR SSF))				!SIM08560
!                                                                       !SIM08570
!    FLAG - INTEGER ERROR FLAG; VALUES AND THEIR MEANINGS ARE:	        !SIM08580
!              0	  NO ERRORS DETECTED.                               !SIM08590
!              2N+K   DUPLICATE ENTRY IN A. ROW = K						!SIM08600
!              6N+K   INSUFFICIENT STORAGE IN SSF. ROW = K				!SIM08610
!              7N+1   INSUFFICIENT STORAGE IN SNF                       !SIM08620
!              8N+K   ZERO PIVOT. ROW = K								!SIM08630
!             10N+1   INSUFFICIENT STORAGE IN SDRV                      !SIM08640
!             11N+1   ILLEGAL PATH SPECIFICATION                        !SIM08650
!                                                                       !SIM08660
!ATTENTION---------------------------------------------------------		!SIM08670
!CONVERSION FROM REAL TO DOUBLE PRECISION: THE REAL DECLARATIONS IN 	!SIM08680
!SDRV, SNF AND SNS WERE CHANGED TO DOUBLE PRECISION. INTEGER VARIA-		!SIM08690
!BLE "RATIO" (IN SDRV) WAS CHANGED FROM 1 TO 2.							!SIM08700
!------------------------------------------------------------------		!SIM08710
!																		!SIM08720
!VARIABLE DECLARATION***************************************************!SIM08730
 INTEGER(K4B), INTENT(IN):: N, NSP, PATH							    !SIM08740
 INTEGER(K4B),INTENT(INOUT)::P(N),IP(N),IA(N+1),JA(*),ISP(NSP),ESP,FLAG !SIM08750
 REAL(DP), INTENT(INOUT):: A(*), B(N), Z(N), RSP(NSP)                   !SIM08760
!REAL, INTENT(INOUT)::  A(*), B(N), Z(N), RSP(NSP)                      !SIM08770
 INTEGER(K4B):: RATIO, Q, MARK, D, U, TMP, JUMAX, UMAX                  !SIM08780
 DATA RATIO/2/															!SIM08790
!DATA:: RATIO/1/														!SIM08800
!***********************************************************************!SIM08810
!                                                                       !SIM08820
!VALIDATE PATH SPECIFICATION											!SIM08830
 IF (PATH.LT.1 .OR. 6.LT.PATH) GOTO 111									!SIM08840
!                                                                       !SIM08850
!ALLOCATE STORAGE AND FACTOR M SYMBOLICALLY TO DETERMINE FILL-IN		!SIM08860
 IJU   = 1																!SIM08870
 IU    = IJU     +  N													!SIM08880
 JL    = IU      + N+1													!SIM08890
 JU    = JL      +  N													!SIM08900
 Q     = (NSP+1) -  N													!SIM08910
 MARK  = Q       -  N													!SIM08920
 JUMAX = MARK    - JU													!SIM08930
!                                                                       !SIM08940
 IF ((PATH-1) * (PATH-4) * (PATH-5) .NE. 0) GOTO 1						!SIM08950
 IF (JUMAX.LE.0) GOTO 110												!SIM08960
!																		!SIM08970
!				                                                        !SIM08980
 CALL SSF (N,P,IP,IA,JA,ISP(IJU),ISP(JU),ISP(IU),JUMAX,ISP(Q),&			!SIM08990
	       ISP(MARK),ISP(JL),FLAG)										!SIM09000
 IF (FLAG.NE.0) GOTO 100			                                    !SIM09010
!                                                                       !SIM09020
!ALLOCATE STORAGE AND FACTOR M NUMERICALLY								!SIM09030
 1  IL   = JU      + ISP(IJU+(N-1))										!SIM09040
    TMP  = ((IL-1)+(RATIO-1)) / RATIO  +  1								!SIM09050
    D    = TMP     + N													!SIM09060
    U    = D       + N													!SIM09070
    UMAX = (NSP+1) - U													!SIM09080
    ESP  = UMAX    - (ISP(IU+N)-1)										!SIM09090
!																		!SIM09100
    IF ((PATH-1) * (PATH-2) * (PATH-5) * (PATH-6) .NE. 0) GOTO 2		!SIM09110
    IF (UMAX.LE.0) GOTO 110												!SIM09120
!																		!SIM09130
    CALL SNF (N, P, IP, IA, JA, A, RSP(D), ISP(IJU), ISP(JU), ISP(IU),&	!SIM09140
		      RSP(U), UMAX, ISP(IL),  ISP(JL),  FLAG)					!SIM09150
!								                                        !SIM09160
    IF (FLAG.NE.0) GOTO 100												!SIM09170
!                                                                       !SIM09180
!SOLVE SYSTEM OF LINEAR EQUATIONS  MX = B								!SIM09190
 2  IF ((PATH-1) * (PATH-2) * (PATH-3) .NE. 0) GOTO 3					!SIM09200
    IF (UMAX.LE.0) GOTO 110												!SIM09210
!																	    !SIM09220
    CALL SNS (N,P,RSP(D),ISP(IJU),ISP(JU),ISP(IU),RSP(U),Z,B,RSP(TMP))  !SIM09230
!                                                                       !SIM09240
 3  RETURN																!SIM09250
!                                                                       !SIM09260
!ERROR SPECIFICATION:							                        !SIM09270
!-------------------												   	!SIM09280
 100  RETURN !ERROR DETECTED IN SSF, SNF, OR SNS						!SIM09290
!																		!SIM09300
 110  FLAG = 10*N + 1  !INSUFFICIENT STORAGE							!SIM09310
      RETURN                                                            !SIM09320
!												                        !SIM09330
 111    FLAG = 11*N + 1 !ILLEGAL PATH SPECIFIED                         !SIM09340
!			                                                            !SIM09350
 END SUBROUTINE SDRV													!SIM09360
!***                                                                    !SIM09370
!                                                                       !SIM09380
!***********************************************************************!SIM09390
!INTERNAL DOCUMENTATION FOR SSF, SNF, AND SNS							!SIM09400
!***********************************************************************!SIM09410
!																		!SIM19420
!-------------------------------------                                  !SIM09430
!COMPRESSED STORAGE OF SPARSE MATRICES									!SIM09440
!-------------------------------------                                  !SIM09450
!THE STRICT UPPER TRIANGULAR PORTION OF THE MATRIX U IS STORED IN (IA,	!SIM09460
!JA,A) FORMAT USING THE ARRAYS IU, JU, AND U, EXCEPT THAT AN ADDITIO-	!SIM09470
!NAL ARRAY IJU IS USED TO REDUCE THE STORAGE REQUIRED FOR JU BY ALLO-  	!SIM09480
!WING SOME SEQUENCES OF COLUMN INDICES TO CORRESPOND TO MORE THAN ONE	!SIM09490
!ROW. FOR I < N, IJU(I) IS THE INDEX IN JU OF THE FIRST ENTRY FOR THE 	!SIM09500
!I-TH ROW; IJU(N) IS THE NUMBER OF ENTRIES IN JU. THUS, THE NUMBER OF	!SIM09510
!ENTRIES IN THE I-TH ROW IS IU(I+1) - IU(I), THE NONZERO ENTRIES OF  	!SIM09520
!THE I-TH ROW ARE STORED CONSECUTIVELY IN:								!SIM09530
!                                                                       !SIM09540
!        U(IU(I)),   U(IU(I)+1),   ..., U(IU(I+1)-1),                   !SIM09550
!                                                                       !SIM09560
!AND THE CORRESPONDING COLUMN INDICES ARE STORED CONSECUTIVELY IN:		!SIM09570
!                                                                       !SIM09580
!        JU(IJU(I)), JU(IJU(I)+1), ..., JU(IJU(I)+IU(I+1)-IU(I)-1).     !SIM09590
!                                                                       !SIM09600
!COMPRESSION IN JU OCCURS IN TWO WAYS. FIRST, IF A ROW I WAS MERGED IN-	!SIM09610
!TO ROW K, AND THE NUMBER OF ELEMENTS MERGED IN FROM (THE TAIL PORTION	!SIM09620
!OF) ROW I IS THE SAME AS THE  FINAL LENGTH OF ROW K, THEN THE KTH ROW 	!SIM09630
!AND THE TAIL OF ROW I ARE IDENTICAL AND IJU(K) POINTS TO THE START OF	!SIM09640
!THE TAIL. SECOND, IF SOME TAIL PORTION OF THE (K-1)ST ROW IS IDENTICAL	!SIM09650
!TO THE HEAD OF THE KTH ROW, THEN IJU(K) POINTS TO THE START OF THAT 	!SIM09660
!TAIL PORTION. FOR EXAMPLE, THE NONZERO	STRUCTURE OF THE STRICT UPPER	!SIM09670
!TRIANGULAR PART OF THE MATRIX:											!SIM09680
!                                                                       !SIM09690
!             ( D 0 0 0 X X X )                                         !SIM09070
!             ( 0 D 0 X X 0 0 )                                         !SIM09710
!             ( 0 0 D 0 X X 0 )                                         !SIM09720
!         U = ( 0 0 0 D X X 0 )                                         !SIM09730
!             ( 0 0 0 0 D X X )                                         !SIM09740
!             ( 0 0 0 0 0 D X )                                         !SIM09750
!             ( 0 0 0 0 0 0 D )                                         !SIM09760
!                                                                       !SIM09770
!WOULD BE STORED AS:													!SIM09780
!                                                                       !SIM09790
!             | 1  2  3  4  5  6  7  8                                  !SIM09800
!         ----+------------------------                                 !SIM09810
!          IU | 1  4  6  8 10 12 13 13                                  !SIM09820
!          JU | 5  6  7  4  5  6                                        !SIM09830
!         IJU | 1  4  5  5  2  3  6           .                         !SIM09840
!                                                                       !SIM09850
!THE DIAGONAL ENTRIES OF U ARE EQUAL TO ONE AND ARE NOT STORED.			!SIM09860
!                                                                       !SIM09870
!---------------------                                                  !SIM09880
!ADDITIONAL PARAMETERS	                                                !SIM09890
!---------------------                                                  !SIM09900
!    D     -  REAL VECTOR CONTAINING THE RECIPROCALS OF THE DIAGONAL	!SIM09910
!             ENTRIES OF THE MATRIX D; DIMENSION = N				    !SIM09920
!                                                                       !SIM09930
!    IJU   -  INTEGER VECTOR CONTAINING POINTERS TO THE START OF EACH	!SIM09940
!             ROW IN JU; DIMENSION = N					                !SIM09950
!                                                                       !SIM09960
!    IU    -  INTEGER VECTOR CONTAINING POINTERS TO DELIMIT ROWS IN U;  !SIM09970
!             DIMENSION = N+1											!SIM09980
!                                                                       !SIM09990
!    JU    -  INTEGER VECTOR CONTAINING THE COLUMN INDICES CORRESPOND-	!SIM10000
!             ING TO THE ELEMENTS OF U; DIMENSION = JUMAX			    !SIM10010
!                                                                       !SIM10020
!    JUMAX -  DECLARED DIMENSION OF THE VECTOR JU; JUMAX MUST BE AT		!SIM10030
!             LEAST THE SIZE OF U MINUS COMPRESSION (IJU(N) AFTER THE	!SIM10040
!             CALL TO SSF)												!SIM10050
!                                                                       !SIM10060
!    U     -  REAL VECTOR CONTAINING THE NONZERO ENTRIES IN THE STRICT  !SIM10070
!             UPPER TRIANGLE OF U, STORED BY ROWS; DIMENSION = UMAX.    !SIM10080
!																		!SIM10090
!    UMAX  -  ECLARED DIMENSION OF THE VECTOR U; UMAX MUST BE AT LEAST  !SIM10100
!             THE NUMBER OF NONZERO ENTRIES IN THE  STRICT UPPER TRIAN- !SIM10110
!             GLE OF M PLUS FILLIN (IU(N+1)-1 AFTER THE CALL TO SSF)	!SIM10120
!                                                                       !SIM10130
!                                                                       !SIM10140
!***********************************************************************!SIM10150
 SUBROUTINE SSF (N,P,IP,IA,JA,IJU,JU,IU,JUMAX,Q,MARK,JL,FLAG)			!SIM10160
!***********************************************************************!SIM10170
!SYMBOLIC UT-D-U FACTORIZATION OF SPARSE SYMMETRIC MATRIX.				!SIM10180
!																		!SIM10190
!---------------------												    !SIM10200
!ADDITIONAL PARAMETERS													!SIM10210
!---------------------                                                  !SIM10220
!    Q    -  INTEGER WORK VECTOR ARRAY; DIMENSION = N			        !SIM10230
!                                                                       !SIM10240
!    MARK -  INTEGER WORK VECTOR ARRAY; DIMENSION = N					!SIM10250
!																        !SIM10260
!    JL   -  INTEGER WORK VECTOR ARRAY; DIMENSION = N					!SIM10270
!                                                                       !SIM10280
!---------------------------------------------------------------------  !SIM10290
!DEFINITIONS OF INTERNAL PARAMETERS (DURING K-TH STAGE OF ELIMINATION)	!SIM10300
!---------------------------------------------------------------------  !SIM10310
! Q CONTAINS AN ORDERED LINKED LIST REPRESENTATION OF THE NONZERO		!SIM10320
!   STRUCTURE OF THE K-TH ROW OF U;										!SIM10330
!		Q(K) IS THE FIRST COLUMN WITH A NONZERO ENTRY					!SIM10340
!		Q(I) IS THE NEXT COLUMN WITH A NONZERO ENTRY AFTER COLUMN I		!SIM10350
! IN EITHER CASE, Q(I) = N+1 INDICATES THE END OF THE LIST.				!SIM10360
!                                                                       !SIM10370
! JL CONTAINS LISTS OF ROWS TO BE MERGED INTO UNELIMINATED ROWS;		!SIM10380
!       I GE K => JL(I) IS THE FIRST ROW TO BE MERGED INTO ROW I		!SIM10390
!       I LT K => JL(I) IS THE ROW FOLLOWING ROW I IN SOME LIST OF ROWS	!SIM10400
! IN EITHER CASE, JL(I) = 0 INDICATES THE END OF A LIST.				!SIM10410
!                                                                       !SIM10420
! MARK(I) IS THE LAST ROW STORED IN JU FOR WHICH U(MARK(I),I) NE 0.		!SIM10430
!                                                                       !SIM10440
! JUMIN AND JUPTR ARE THE INDICES IN JU OF THE FIRST AND LAST			!SIM10450
!		ELEMENTS IN THE LAST ROW SAVED IN JU.                           !SIM10460
!                                                                       !SIM10470
! LUK IS THE NUMBER OF NONZERO ENTRIES IN THE K-TH ROW.					!SIM10480
!                                                                       !SIM10490
!VARIABLE DECLARATION***************************************************!SIM10500
 INTEGER(K4B):: P(N), IP(N), IA(N+1), JA(*), IJU(N), IU(N+1), JU(*)		!SIM10510
 INTEGER(K4B):: Q(N), MARK(N), JL(N), FLAG, TAG, VJ, QM					!SIM10520
 LOGICAL:: CLIQUE														!SIM10530
!***********************************************************************!SIM10540
!                                                                       !SIM10550
!	INITIALIZATION														!SIM10560
	JUMIN = 1															!SIM10570
	JUPTR = 0															!SIM10580
	IU(1) = 1															!SIM10590
	DO 1 K=1,N															!SIM10600
		MARK(K) = 0														!SIM10610
 1		JL(K) = 0														!SIM10620
!                                                                       !SIM10630
!	FOR EACH ROW K														!SIM10640
	DO 18 K=1,N															!SIM10650
		LUK = 0															!SIM10660
		Q(K) = N+1														!SIM10670
!                                                                       !SIM10680
		TAG = MARK(K)													!SIM10690
		CLIQUE = .FALSE.												!SIM10700
		IF (JL(K).NE.0) CLIQUE = JL(JL(K)).EQ.0							!SIM10710
!                                                                       !SIM10720
!		INITIALIZE NONZERO STRUCTURE OF K-TH ROW TO ROW P(K) OF M       !SIM10730
		JMIN = IA(P(K))                                                 !SIM10740
		JMAX = IA(P(K)+1) - 1                                           !SIM10750
		IF (JMIN.GT.JMAX) GOTO 4										!SIM10760
		DO 3 J=JMIN,JMAX                                                !SIM10770
			VJ = IP(JA(J))                                              !SIM10780
			IF (VJ.LE.K) GOTO 3                                         !SIM10790
!                                                                       !SIM10800
			QM = K                                                      !SIM10810
 2			 M = QM                                                     !SIM10820
			QM = Q(M)                                                   !SIM10830
			IF (QM.LT.VJ) GOTO 2                                        !SIM10840
			IF (QM.EQ.VJ) GOTO 102									    !SIM10850
			LUK = LUK+1                                                 !SIM10860
			Q(M) = VJ                                                   !SIM10870
			Q(VJ) = QM                                                  !SIM10880
			IF (MARK(VJ).NE.TAG) CLIQUE = .FALSE.                       !SIM10890
!                                                                       !SIM10900
 3		CONTINUE														!SIM10910
!                                                                       !SIM10920
!		IF EXACTLY ONE ROW IS TO BE MERGED INTO THE K-TH ROW AND THERE	!SIM10930
!		IS A NZ ENTRY IN EVERY COLUMN IN THAT ROW IN WHICH THERE'S A NZ !SIM10940
!		ENTRY IN ROW P(K) OF M, THEN DO NOT COMPUTE FILL-IN. JUST USE	!SIM10950
!		THE COLUMN INDICES FOR THE ROW WHICH WAS TO HAVE BEEN MERGED.	!SIM10960
 4      IF (.NOT.CLIQUE) GOTO 5											!SIM10970
        IJU(K) = IJU(JL(K)) + 1											!SIM10980
        LUK = IU(JL(K)+1) - (IU(JL(K))+1)								!SIM10990
        GOTO 17															!SIM11000
!                                                                       !SIM11010
!		MODIFY NONZERO STRUCTURE OF K-TH ROW BY COMPUTING FILL-IN FOR   !SIM11020
!		EACH ROW I TO BE MERGED IN.										!SIM11030
 5      LMAX = 0														!SIM11040
        IJU(K) = JUPTR                                                  !SIM11050
!                                                                       !SIM11060
        I = K                                                           !SIM11070
 6		I = JL(I)							                            !SIM11080
        IF (I.EQ.0) GOTO 10				                                !SIM11090
!                                                                       !SIM11100
!		MERGE ROW I INTO K-TH ROW.					                    !SIM11110
        LUI = IU(I+1) - (IU(I)+1)				                        !SIM11120
        JMIN = IJU(I) +  1			                                    !SIM11130
        JMAX = IJU(I) + LUI					                            !SIM11140
        QM = K									                        !SIM11150
!													                    !SIM11160
        DO 8 J=JMIN,JMAX								                !SIM11170
			VJ = JU(J)                                                  !SIM11180
 7	        M = QM											            !SIM11190
            QM = Q(M)                                                   !SIM11200
            IF (QM.LT.VJ) GOTO 7									    !SIM11210
            IF (QM.EQ.VJ) GOTO 8								        !SIM11220
			LUK = LUK+1                                                 !SIM11230
			Q(M) = VJ                                                   !SIM11240
			Q(VJ) = QM                                                  !SIM11250
			QM = VJ                                                     !SIM11260
 8		CONTINUE		                                                !SIM11270
!                                                                       !SIM11280
!		REMEMBER LENGTH AND POSITION IN JU OF LONGEST ROW MERGED.		!SIM11290
        IF (LUI.LE.LMAX) GOTO 9											!SIM11300
        LMAX = LUI														!SIM11310
        IJU(K) = JMIN													!SIM11320
!                                                                       !SIM11330
 9	    GOTO 6															!SIM11340
!                                                                       !SIM11350
!		IF THE K-TH ROW IS THE SAME LENGTH AS THE LONGEST ROW MERGED,   !SIM11360
!		THEN USE THE COLUMN INDICES FOR THAT ROW.                       !SIM11370
  10    IF (LUK.EQ.LMAX) GOTO 17										!SIM11380
!                                                                       !SIM11390
!		IF THE TAIL OF THE LAST ROW SAVED IN JU IS THE SAME AS THE HEAD !SIM11400
!		OF THE K-TH ROW, THEN OVERLAP THE TWO SETS OF COLUMN INDICES.	!SIM11410
!		SEARCH LAST ROW SAVED FOR FIRST NONZERO ENTRY IN K-TH ROW...	!SIM11420
        I = Q(K)														!SIM11430
        IF (JUMIN.GT.JUPTR) GOTO 12										!SIM11440
        DO 11 JMIN=JUMIN,JUPTR											!SIM11450
			IF (JU(JMIN)-I)  11, 13, 12                                 !SIM11460
 11     CONTINUE														!SIM11470
 12     GOTO 15															!SIM11480
!                                                                       !SIM11490
!		...AND THEN TEST WHETHER TAIL MATCHES HEAD OF K-TH ROW.			!SIM11500
 13     IJU(K) = JMIN													!SIM11510
        DO 14 J=JMIN,JUPTR												!SIM11520
			IF (JU(J).NE.I) GOTO 15										!SIM11530
            I = Q(I)                                                    !SIM11540
            IF (I.GT.N) GOTO 17											!SIM11550
 14     CONTINUE														!SIM11560
        JUPTR = JMIN - 1												!SIM11570
!                                                                       !SIM11580
!		SAVE NONZERO STRUCTURE OF K-TH ROW IN JU.                       !SIM11590
 15     I = K															!SIM11600
        JUMIN = JUPTR +  1                                              !SIM11610
        JUPTR = JUPTR + LUK                                             !SIM11620
        IF (JUPTR.GT.JUMAX) GOTO 106									!SIM11630
        DO 16 J=JUMIN,JUPTR												!SIM11640
			I = Q(I)													!SIM11650
			JU(J) = I													!SIM11660
 16			MARK(I) = K                                                 !SIM11670
        IJU(K) = JUMIN                                                  !SIM11680
!                                                                       !SIM11690
!		ADD K TO ROW LIST FOR FIRST NONZERO ELEMENT IN K-TH ROW.        !SIM11700
 17     IF (LUK.LE.1) GOTO 18											!SIM11710
        I = JU(IJU(K))													!SIM11720
        JL(K) = JL(I)													!SIM11730
        JL(I) = K														!SIM11740
!                                                                       !SIM11750
 18     IU(K+1) = IU(K) + LUK											!SIM11760
!                                                                       !SIM11770
	FLAG = 0															!SIM11780
    RETURN																!SIM11790
!											                            !SIM11800
!ERROR SPECIFICATION:				                                    !SIM11810
 102  FLAG = 2*N + P(K) !DUPLICATE ENTRY IN A                           !SIM11820
      RETURN                                                            !SIM11830
!																		!SIM11840
 106  FLAG = 6*N + K  !INSUFFICIENT STORAGE FOR JU                      !SIM11850
!			                                                            !SIM11860
 END SUBROUTINE SSF                                                     !SIM11870
!                                                                       !SIM11880
!***********************************************************************!SIM11890
 SUBROUTINE SNF (N,P,IP,IA,JA,A,D,IJU,JU,IU,U,UMAX,IL,JL,FLAG)			!SIM11900
!***********************************************************************!SIM11910
!NUMERICAL UT-D-U FACTORIZATION OF SPARSE SYMMETRIC POSITIVE DEFINITE	!SIM11920
!MATRIX.																!SIM11930
!															            !SIM11940
!---------------------                                                  !SIM11950
!ADDITIONAL PARAMETERS		                                            !SIM11960
!---------------------                                                  !SIM11970
!    IL - INTEGER ONE-DIMENSIONAL WORK ARRAY;  DIMENSION = N			!SIM11980
!                                                                       !SIM11990
!    JL - INTEGER ONE-DIMENSIONAL WORK ARRAY;  DIMENSION = N			!SIM12000
!                                                                       !SIM12010
!---------------------------------------------------------------------	!SIM12020
!DEFINITIONS OF INTERNAL PARAMETERS (DURING K-TH STAGE OF ELIMINATION)	!SIM12030
!---------------------------------------------------------------------	!SIM12040
! (D(I),I=K,N) CONTAINS THE K-TH ROW OF U (EXPANDED).					!SIM12050
!                                                                       !SIM12060
! IL(I) POINTS TO THE FIRST NONZERO ELEMENT IN COLUMNS K,...,N OF		!SIM12070
!       ROW I OF U.                                                     !SIM12080
!                                                                       !SIM12090
! JL CONTAINS LISTS OF ROWS TO BE ADDED TO UNELIMINATED ROWS.			!SIM12100
!		I GE K => JL(I) IS THE FIRST ROW TO BE ADDED TO ROW I           !SIM12110
!		I LT K => JL(I) IS THE ROW FOLLOWING ROW I IN SOME LIST OF ROWS !SIM12120
!    IN EITHER CASE, JL(I) = 0 INDICATES THE END OF A LIST.				!SIM12130
!                                                                       !SIM12140
!VARIABLE DECLARATION***************************************************!SIM12150
 INTEGER(K4B):: P(N), IP(N), IA(N+1), JA(*), IJU(N), JU(*), IU(N+1)		!SIM12160
 INTEGER(K4B):: UMAX,  IL(N),  JL(N),  FLAG,  VJ						!SIM12170
!REAL::  A(*),  D(N), U(UMAX),  DK, UKIDI								!SIM12180
 REAL(DP):: A(*), D(N), U(UMAX), DK, UKIDI						        !SIM12190
!***********************************************************************!SIM12200
!                                                                       !SIM12210
!CHECK FOR SUFFICIENT STORAGE FOR U										!SIM12220
 IF (IU(N+1)-1 .GT. UMAX) GOTO 107										!SIM12230
!                                                                       !SIM12240
!	INITIALIZATION														!SIM12250
	DO 1 K=1,N															!SIM12260
         D(K) = 0                                                       !SIM12270
 1      JL(K) = 0														!SIM12280
!                                                                       !SIM12290
!	FOR EACH ROW K														!SIM12300
    DO 11 K=1,N															!SIM12310
!                                                                       !SIM12320
!		INITIALIZE K-TH ROW WITH ELEMENTS NONZERO IN ROW P(K) OF M.     !SIM12330
 3      JMIN = IA(P(K))													!SIM12340
        JMAX = IA(P(K)+1) - 1                                           !SIM12350
        IF (JMIN.GT.JMAX) GOTO 5										!SIM12360
        DO 4 J=JMIN,JMAX                                                !SIM12370
			VJ = IP(JA(J))                                              !SIM12380
			IF (K.LE.VJ) D(VJ) = A(J)	                                !SIM12390
 4      CONTINUE								                        !SIM12400
!                                                                       !SIM12410
!		MODIFY K-TH ROW BY ADDING IN THOSE ROWS I WITH U(I,K) NE 0 FOR  !SIM12420
!		FOR EACH ROW I TO BE ADDED IN.									!SIM12430
 5      DK = D(K)														!SIM12440
        I = JL(K)                                                       !SIM12450
 6      IF (I.EQ.0) GOTO 9												!SIM12460
        NEXTI = JL(I)	                                                !SIM12470
!                                                                       !SIM12480
!		COMPUTE MULTIPLIER AND UPDATE DIAGONAL ELEMENT.                 !SIM12490
        ILI = IL(I)														!SIM12500
        UKIDI = - U(ILI) * D(I)											!SIM12510
        DK = DK + UKIDI * U(ILI)										!SIM12520
        U(ILI) = UKIDI													!SIM12530
!                                                                       !SIM12540
!		ADD MULTIPLE OF ROW I TO K-TH ROW...	                        !SIM12550
        JMIN = ILI     + 1												!SIM12560
        JMAX = IU(I+1) - 1												!SIM12570
        IF (JMIN.GT.JMAX) GOTO 8					                    !SIM12580
        MU = IJU(I) - IU(I)												!SIM12590
        DO 7 J=JMIN,JMAX												!SIM12600
 7			D(JU(MU+J)) = D(JU(MU+J)) + UKIDI * U(J)					!SIM12610
!                                                                       !SIM12620
!		...AND ADD I TO ROW LIST FOR NEXT NONZERO ENTRY					!SIM12630
        IL(I) = JMIN													!SIM12640
        J = JU(MU+JMIN)													!SIM12650
        JL(I) = JL(J)													!SIM12660
        JL(J) = I														!SIM12670
!                                                                       !SIM12680
 8      I = NEXTI														!SIM12690
        GOTO 6															!SIM12700
!                                                                       !SIM12710
!		CHECK FOR ZERO PIVOT AND SAVE DIAGONAL ELEMENT					!SIM12720
 9      IF (DK.EQ.0) GOTO 108											!SIM12730
        D(K) = 1 / DK                                                   !SIM12740
!                                                                       !SIM12750
!		SAVE NONZERO ENTRIES IN K-TH ROW OF U...	                    !SIM12760
        JMIN = IU(K)                                                    !SIM12770
        JMAX = IU(K+1) - 1                                              !SIM12780
        IF (JMIN.GT.JMAX) GOTO 11										!SIM12790
        MU = IJU(K) - JMIN												!SIM12800
        DO 10 J=JMIN,JMAX												!SIM12810
			JUMUJ = JU(MU+J)                                            !SIM12820
            U(J) = D(JUMUJ)                                             !SIM12830
10          D(JUMUJ) = 0												!SIM12840
!                                                                       !SIM12850
!		...AND ADD K TO ROW LIST FOR FIRST NONZERO ENTRY IN K-TH ROW.   !SIM12860
        IL(K) = JMIN													!SIM12870
        I = JU(MU+JMIN)													!SIM12880
        JL(K) = JL(I)													!SIM12890
        JL(I) = K								                        !SIM12900
11	CONTINUE															!SIM12910
!                                                                       !SIM12920
    FLAG = 0									                        !SIM12930
    RETURN											                    !SIM12940
!                                                                       !SIM12950
!ERROR SPECIFICATION:						                            !SIM12960
 107  FLAG = 7*N + 1  !INSUFFICIENT STORAGE FOR U                       !SIM12970
      RETURN                                                            !SIM12980
!											                            !SIM12990
 108  FLAG = 8*N + K !ZERO PIVOT						                !SIM13000
!			                                                            !SIM13010
 END SUBROUTINE SNF														!SIM13020
!                                                                       !SIM13030
!***********************************************************************!SIM13040
 SUBROUTINE SNS (N, P, D, IJU, JU, IU, U, Z, B, TMP)                    !SIM13050
!***********************************************************************!SIM13060
!SOLUTION OF SPARSE SYMMETRIC POSITIVE DEFINITE SYSTEM OF LINEAR EQUA-  !SIM13070
!TIONS MX = B GIVEN UT-D-U FACTORIZATION OF M.							!SIM13080
!                                                                       !SIM13090
!---------------------													!SIM13100
!ADDITIONAL PARAMETERS													!SIM13110
!---------------------                                                  !SIM13120
!TMP - REAL ONE-DIMENSIONAL WORK ARRAY; DIMENSION = N					!SIM13130
!                                                                       !SIM13140
!VARIABLE DECLARATION***************************************************!SIM13150
 INTEGER(K4B)::  P(N),  IJU(N), JU(*), IU(N+1)							!SIM13160
!REAL::  D(N), U(UMAX), Z(N), B(N), TMP(N), TMPK, SUM					!SIM13170
 REAL(DP):: D(1), U(1), Z(N), B(N), TMP(N), TMPK, SUM		        	!SIM13180
!***********************************************************************!SIM13190
!					                                                    !SIM13200
!   SET TMP TO PERMUTED B												!SIM13210
	DO 1 K=1,N															!SIM13220
 1      TMP(K) = B(P(K))												!SIM13230
!                                                                       !SIM13240
!   SOLVE UT D Y = B BY FORWARD SUBSTITUTION.							!SIM13250
    DO 3 K=1,N								                            !SIM13260
		TMPK = TMP(K)                                                   !SIM13270
        JMIN = IU(K)                                                    !SIM13280
        JMAX = IU(K+1) - 1                                              !SIM13290
        IF (JMIN.GT.JMAX) GOTO 3				                        !SIM13300
        MU = IJU(K) - JMIN                                              !SIM13310
        DO 2 J=JMIN,JMAX                                                !SIM13320
 2			TMP(JU(MU+J)) = TMP(JU(MU+J)) + U(J) * TMPK                 !SIM13330
 3      TMP(K) = TMPK * D(K)						                    !SIM13340
!                                                                       !SIM13350
!	SOLVE U X = Y BY BACK SUBSTITUTION									!SIM13360
    K = N																!SIM13370
    DO 6 I=1,N														    !SIM13380
		SUM = TMP(K)                                                    !SIM13390
        JMIN = IU(K)                                                    !SIM13400
        JMAX = IU(K+1) - 1                                              !SIM13410
        IF (JMIN.GT.JMAX) GOTO 5										!SIM13420
        MU = IJU(K) - JMIN                                              !SIM13430
        DO 4 J=JMIN,JMAX                                                !SIM13440
 4			SUM = SUM + U(J) * TMP(JU(MU+J))							!SIM13450
 5      TMP(K) = SUM													!SIM13460
        Z(P(K)) = SUM                                                   !SIM13470
 6      K = K-1															!SIM13480
!			                                                            !SIM13490
 END SUBROUTINE SNS                                                     !SIM13500
!***********************************************************************

 END MODULE SIM
!***